@@ -92,9 +92,6 @@
 
 #include	<juman.h>
 
-/* #define DEBUG_MODE */
-/* #define ALL_DEBUG_MODE */
-
 /*
 ------------------------------------------------------------------------------
 	definition of macros
@@ -156,6 +153,7 @@ int             Show_Opt1;
 int             Show_Opt2;
 char		Show_Opt_tag[MIDASI_MAX];
 int		Show_Opt_jumanrc;
+int		Show_Opt_debug;
 
 U_CHAR	        String[LENMAX];
 int             m_buffer_num;
@@ -632,13 +630,15 @@ int take_data(int pos, char **pbuf)
 		    }
 /*printf("  TO %d\n", new_mrph->con_tbl);*/
 		    pnum_bak = p_buffer_num;
-#ifdef DEBUG_MODE
-		    printf("\\begin{ǢϢ} %s", mrph.midasi);
-		    if (add_list[co])
-			printf("%s",
-			   Form[mrph_buf[num-1].katuyou1][add_list[co]].gobi);
-		    printf("\n");
-#endif
+
+		    if (Show_Opt_debug) {
+			printf("\\begin{ǢϢ} %s", mrph.midasi);
+			if (add_list[co])
+			    printf("%s",
+				   Form[mrph_buf[num-1].katuyou1][add_list[co]].gobi);
+			printf("\n");
+		    }
+		    
 		    check_connect(pos, m_buffer_num);
 		    if (p_buffer_num == pnum_bak) break;
 
@@ -649,10 +649,11 @@ int take_data(int pos, char **pbuf)
 			 * new_mrph->weight * cost_omomi.keitaiso 
 			 * (rengo_weight-10)/10);
 		    new_mrph->con_tbl = con_tbl_bak;
-#ifdef DEBUG_MODE
-		    printf("----- Ϣ %s %d\n", new_mrph->midasi, 
-			   p_buffer[pnum_bak].score);
-#endif
+
+		    if (Show_Opt_debug) {
+			printf("----- Ϣ %s %d\n", new_mrph->midasi, 
+			       p_buffer[pnum_bak].score);
+		    }
 		} else {
 		    p_buffer[p_buffer_num].score =
 			p_buffer[p_buffer_num-1].score +
@@ -691,10 +692,11 @@ int take_data(int pos, char **pbuf)
 			    p_buffer[p_buffer_num].end += glen;
 			}
 		    }
-#ifdef DEBUG_MODE
-		    printf("----- Ϣ %s %d\n", new_mrph->midasi, 
-			   p_buffer[p_buffer_num].score);
-#endif
+		    if (Show_Opt_debug) {
+			printf("----- Ϣ %s %d\n", new_mrph->midasi, 
+			       p_buffer[p_buffer_num].score);
+		    }
+
 		    if (++p_buffer_num == process_buffer_max)
 			realloc_process_buffer();
 		}
@@ -706,9 +708,9 @@ int take_data(int pos, char **pbuf)
 		if (++m_buffer_num == mrph_buffer_max)
 		  realloc_mrph_buffer();
 	    }
-#ifdef DEBUG_MODE
-	    printf("\\end{ǢϢ}\n");
-#endif
+	    if (Show_Opt_debug) {
+		printf("\\end{ǢϢ}\n");
+	    }
 	}
 
     } else {           /* ̤ηǤä */
@@ -1281,7 +1283,6 @@ void print_path_mrph(FILE* output, int path_num , int para_flag)
     MRPH       	*mrph_p;
     int		newDicNo;
     int         now_r_buffer_num;
-    MRPH       	*r_mrph;
     MRPH        r_last_mrph;
     int		pos;
     int        	i, j, k, len;
@@ -1641,7 +1642,7 @@ int check_connect(int pos, int m_num)
     int	score, best_score, haba_score, best_score_num;
     int c_score, class_score;
     MRPH *new_mrph;
-    MRPH *r_mrph;
+    MRPH *pre_mrph;
     int left_con, right_con;
     CONNECT_COST *c_cache;
 
@@ -1654,20 +1655,20 @@ int check_connect(int pos, int m_num)
 
     /* Ϣܥå˥ҥåȤСõԤʤ */
     c_cache = &connect_cache[rensetu_tbl[new_mrph->con_tbl].j_pos];
-#ifndef DEBUG_MODE
-    if (c_cache->pos == pos && c_cache->p_no > 0) {
-	p_buffer[p_buffer_num].score = c_cache->cost + class_score;
-	p_buffer[p_buffer_num].mrph_p = m_num;
-	p_buffer[p_buffer_num].start = pos;
-	p_buffer[p_buffer_num].end = pos + new_mrph->length;
-	for (i = 0; (p_buffer[p_buffer_num].path[i] = p_buffer[c_cache->p_no].path[i]) >= 0; i++) {}
-	p_buffer[p_buffer_num].path[i] = -1;
-	p_buffer[p_buffer_num].connect = TRUE;
-	if (++p_buffer_num == process_buffer_max)
-	    realloc_process_buffer();
-	return TRUE;
+    if (Show_Opt_debug == 0) {
+	if (c_cache->pos == pos && c_cache->p_no > 0) {
+	    p_buffer[p_buffer_num].score = c_cache->cost + class_score;
+	    p_buffer[p_buffer_num].mrph_p = m_num;
+	    p_buffer[p_buffer_num].start = pos;
+	    p_buffer[p_buffer_num].end = pos + new_mrph->length;
+	    for (i = 0; (p_buffer[p_buffer_num].path[i] = p_buffer[c_cache->p_no].path[i]) >= 0; i++) {}
+	    p_buffer[p_buffer_num].path[i] = -1;
+	    p_buffer[p_buffer_num].connect = TRUE;
+	    if (++p_buffer_num == process_buffer_max)
+		realloc_process_buffer();
+	    return TRUE;
+	}
     }
-#endif
 
     for (i = 0; (j = match_pbuf[i]) >= 0; i++) {
 
@@ -1692,39 +1693,59 @@ int check_connect(int pos, int m_num)
 	    chk_con_num++;
 	}
 
-#ifdef DEBUG_MODE
-	/* ǥХɽ */
-	while (1) {
-	    MRPH mrph1,*r_mrph;
+	/* ǥХѥɽ */
 
-	    mrph1 = m_buffer[p_buffer[j].mrph_p];
-	    if (c_score == 0) 
-#ifdef ALL_DEBUG_MODE
-	      printf("XXX ");
-#else
-	      break;
-#endif
-	    printf("%s" , mrph1.midasi);
-	    if (Class[mrph1.hinsi][mrph1.bunrui].kt)
-	      printf("%s",Form[mrph1.katuyou1][mrph1.katuyou2].gobi);
-	    printf("(%s" , Class[mrph1.hinsi][0].id);
-	    if (mrph1.bunrui)
-	      printf("-%s",Class[mrph1.hinsi][mrph1.bunrui].id);
-	    mrph1 = *new_mrph;
-	    printf(") %s" , mrph1.midasi);
-	    if (Class[mrph1.hinsi][mrph1.bunrui].kt)
-	      printf("%s",Form[mrph1.katuyou1][mrph1.katuyou2].gobi);
-	    printf("(%s" , Class[mrph1.hinsi][0].id);
-	    if (mrph1.bunrui)
-	      printf("-%s",Class[mrph1.hinsi][mrph1.bunrui].id);
-	    printf(")");
-	    if (c_score == 0) {printf("\n"); break;}
-	    printf(" %d %d  %d %d\n",p_buffer[j].score,score+class_score,
-		   class_score * cost_omomi.keitaiso/10, c_score);
-	    break;
+	if (Show_Opt_debug == 2 || (Show_Opt_debug == 1 && c_score)) {
+
+	    fprintf(stderr, "%3d " , pos);
+
+	    pre_mrph = &m_buffer[p_buffer[j].mrph_p];
+	    fprintf(stderr, "%s" , pre_mrph->midasi);
+	    if (Class[pre_mrph->hinsi][pre_mrph->bunrui].kt)
+		fprintf(stderr, "%s", Form[pre_mrph->katuyou1][pre_mrph->katuyou2].gobi);
+	    if (Class[pre_mrph->hinsi][0].id) {
+		fprintf(stderr, "(%s" , Class[pre_mrph->hinsi][0].id);
+		if (pre_mrph->bunrui)
+		    fprintf(stderr, "-%s", Class[pre_mrph->hinsi][pre_mrph->bunrui].id);
+		if (pre_mrph->katuyou1)
+		    fprintf(stderr, "-%s", Type[pre_mrph->katuyou1].name);
+		if (pre_mrph->katuyou2)
+		    fprintf(stderr, "-%s", Form[pre_mrph->katuyou1][pre_mrph->katuyou2].name);
+		fprintf(stderr, ")");
+	    }
+	    fprintf(stderr, "[= %d]", p_buffer[j].score);
+
+	    if (c_score) {
+		fprintf(stderr, "--[+%d*%d]--", c_score, cost_omomi.rensetsu);
+	    } else {
+		fprintf(stderr, "--XXX--");
+	    }
+
+	    fprintf(stderr, "%s" , new_mrph->midasi);
+	    if (Class[new_mrph->hinsi][new_mrph->bunrui].kt)
+		fprintf(stderr, "%s", Form[new_mrph->katuyou1][new_mrph->katuyou2].gobi);
+	    if (Class[new_mrph->hinsi][0].id) {
+		fprintf(stderr, "(%s" , Class[new_mrph->hinsi][0].id);
+		if (new_mrph->bunrui)
+		    fprintf(stderr, "-%s",Class[new_mrph->hinsi][new_mrph->bunrui].id);
+		if (new_mrph->katuyou1)
+		    fprintf(stderr, "-%s", Type[new_mrph->katuyou1].name);
+		if (new_mrph->katuyou2)
+		    fprintf(stderr, "-%s", Form[new_mrph->katuyou1][new_mrph->katuyou2].name);
+		fprintf(stderr, ")");
+	    }
+
+	    if (c_score == 0) {
+		fprintf(stderr, "\n");
+	    } else {
+		fprintf(stderr, "[+%d*%d.%d*%d = %d]\n", 
+			Class[new_mrph->hinsi][new_mrph->bunrui].cost,
+			new_mrph->weight/10, new_mrph->weight%10, 
+			cost_omomi.keitaiso*10, 
+			/* = class_score */
+			p_buffer[j].score + c_score * cost_omomi.rensetsu + class_score);
+	    }
 	}
-#endif
-
     }
 
     /* return immidiately, because if best_score is
